from solid2.core.object_base import OpenSCADObject, OpenSCADConstant
from solid2.core.scad_import import extra_scad_include
from pathlib import Path

extra_scad_include(f"{Path(__file__).parent.parent / '../libs/BOSL2/comparisons.scad'}", use_not_include=False)

class approx(OpenSCADObject):
    def __init__(self, a=None, b=None, eps=None, **kwargs):
       super().__init__("approx", {"a" : a, "b" : b, "eps" : eps, **kwargs})

class all_zero(OpenSCADObject):
    def __init__(self, x=None, eps=None, **kwargs):
       super().__init__("all_zero", {"x" : x, "eps" : eps, **kwargs})

class all_nonzero(OpenSCADObject):
    def __init__(self, x=None, eps=None, **kwargs):
       super().__init__("all_nonzero", {"x" : x, "eps" : eps, **kwargs})

class all_positive(OpenSCADObject):
    def __init__(self, x=None, eps=None, **kwargs):
       super().__init__("all_positive", {"x" : x, "eps" : eps, **kwargs})

class all_negative(OpenSCADObject):
    def __init__(self, x=None, eps=None, **kwargs):
       super().__init__("all_negative", {"x" : x, "eps" : eps, **kwargs})

class all_nonpositive(OpenSCADObject):
    def __init__(self, x=None, eps=None, **kwargs):
       super().__init__("all_nonpositive", {"x" : x, "eps" : eps, **kwargs})

class all_nonnegative(OpenSCADObject):
    def __init__(self, x=None, eps=None, **kwargs):
       super().__init__("all_nonnegative", {"x" : x, "eps" : eps, **kwargs})

class all_equal(OpenSCADObject):
    def __init__(self, vec=None, eps=None, **kwargs):
       super().__init__("all_equal", {"vec" : vec, "eps" : eps, **kwargs})

class is_increasing(OpenSCADObject):
    def __init__(self, list=None, strict=None, **kwargs):
       super().__init__("is_increasing", {"list" : list, "strict" : strict, **kwargs})

class is_decreasing(OpenSCADObject):
    def __init__(self, list=None, strict=None, **kwargs):
       super().__init__("is_decreasing", {"list" : list, "strict" : strict, **kwargs})

class _type_num(OpenSCADObject):
    def __init__(self, x=None, **kwargs):
       super().__init__("_type_num", {"x" : x, **kwargs})

class compare_vals(OpenSCADObject):
    def __init__(self, a=None, b=None, **kwargs):
       super().__init__("compare_vals", {"a" : a, "b" : b, **kwargs})

class compare_lists(OpenSCADObject):
    def __init__(self, a=None, b=None, **kwargs):
       super().__init__("compare_lists", {"a" : a, "b" : b, **kwargs})

class min_index(OpenSCADObject):
    def __init__(self, vals=None, all=None, **kwargs):
       super().__init__("min_index", {"vals" : vals, "all" : all, **kwargs})

class max_index(OpenSCADObject):
    def __init__(self, vals=None, all=None, **kwargs):
       super().__init__("max_index", {"vals" : vals, "all" : all, **kwargs})

class find_approx(OpenSCADObject):
    def __init__(self, val=None, list=None, start=None, all=None, eps=None, **kwargs):
       super().__init__("find_approx", {"val" : val, "list" : list, "start" : start, "all" : all, "eps" : eps, **kwargs})

class __find_approx(OpenSCADObject):
    def __init__(self, val=None, list=None, eps=None, i=None, **kwargs):
       super().__init__("__find_approx", {"val" : val, "list" : list, "eps" : eps, "i" : i, **kwargs})

class deduplicate(OpenSCADObject):
    def __init__(self, list=None, closed=None, eps=None, **kwargs):
       super().__init__("deduplicate", {"list" : list, "closed" : closed, "eps" : eps, **kwargs})

class deduplicate_indexed(OpenSCADObject):
    def __init__(self, list=None, indices=None, closed=None, eps=None, **kwargs):
       super().__init__("deduplicate_indexed", {"list" : list, "indices" : indices, "closed" : closed, "eps" : eps, **kwargs})

class unique(OpenSCADObject):
    def __init__(self, list=None, **kwargs):
       super().__init__("unique", {"list" : list, **kwargs})

class _unique_sort(OpenSCADObject):
    def __init__(self, l=None, **kwargs):
       super().__init__("_unique_sort", {"l" : l, **kwargs})

class unique_count(OpenSCADObject):
    def __init__(self, list=None, **kwargs):
       super().__init__("unique_count", {"list" : list, **kwargs})

class _valid_idx(OpenSCADObject):
    def __init__(self, idx=None, imin=None, imax=None, **kwargs):
       super().__init__("_valid_idx", {"idx" : idx, "imin" : imin, "imax" : imax, **kwargs})

class _group_sort_by_index(OpenSCADObject):
    def __init__(self, l=None, idx=None, **kwargs):
       super().__init__("_group_sort_by_index", {"l" : l, "idx" : idx, **kwargs})

class _group_sort(OpenSCADObject):
    def __init__(self, l=None, **kwargs):
       super().__init__("_group_sort", {"l" : l, **kwargs})

class _sort_scalars(OpenSCADObject):
    def __init__(self, arr=None, **kwargs):
       super().__init__("_sort_scalars", {"arr" : arr, **kwargs})

class _sort_vectors(OpenSCADObject):
    def __init__(self, arr=None, _i=None, **kwargs):
       super().__init__("_sort_vectors", {"arr" : arr, "_i" : _i, **kwargs})

class _sort_vectors(OpenSCADObject):
    def __init__(self, arr=None, idxlist=None, _i=None, **kwargs):
       super().__init__("_sort_vectors", {"arr" : arr, "idxlist" : idxlist, "_i" : _i, **kwargs})

class _sort_general(OpenSCADObject):
    def __init__(self, arr=None, idx=None, indexed=None, **kwargs):
       super().__init__("_sort_general", {"arr" : arr, "idx" : idx, "indexed" : indexed, **kwargs})

class _lexical_sort(OpenSCADObject):
    def __init__(self, arr=None, **kwargs):
       super().__init__("_lexical_sort", {"arr" : arr, **kwargs})

class _indexed_sort(OpenSCADObject):
    def __init__(self, arrind=None, **kwargs):
       super().__init__("_indexed_sort", {"arrind" : arrind, **kwargs})

class sort(OpenSCADObject):
    def __init__(self, list=None, idx=None, **kwargs):
       super().__init__("sort", {"list" : list, "idx" : idx, **kwargs})

class sortidx(OpenSCADObject):
    def __init__(self, list=None, idx=None, **kwargs):
       super().__init__("sortidx", {"list" : list, "idx" : idx, **kwargs})

class group_sort(OpenSCADObject):
    def __init__(self, list=None, idx=None, **kwargs):
       super().__init__("group_sort", {"list" : list, "idx" : idx, **kwargs})

class group_data(OpenSCADObject):
    def __init__(self, groups=None, values=None, **kwargs):
       super().__init__("group_data", {"groups" : groups, "values" : values, **kwargs})

class list_smallest(OpenSCADObject):
    def __init__(self, list=None, k=None, **kwargs):
       super().__init__("list_smallest", {"list" : list, "k" : k, **kwargs})




from types import SimpleNamespace
import inspect

from .utils import resolve_scad_filename, escape_openscad_identifier
from .parse_scad import get_scad_file_as_dict

# ===========================
# = IMPORTING OPENSCAD CODE =
# ===========================
module_cache_by_resolved_filename = {}
extra_scad_includes = []

def check_module_cache(resolved_scad, use_not_include):
    global module_cache_by_resolved_filename

    #check the module cache for the requested module
    if resolved_scad in module_cache_by_resolved_filename.keys():
        cache_entry = module_cache_by_resolved_filename[resolved_scad]

        #if this is a include call and the module was only used until now
        #set the "use/include" flag in the module cache
        if not use_not_include and cache_entry[1]:
            cache_entry[1] = False

        return cache_entry[0]

    return None

def update_module_cache(resolved_scad, new_namespace_dict, use_not_include, skip_render):
    global module_cache_by_resolved_filename
    module_cache_by_resolved_filename[resolved_scad] = \
                                        [new_namespace_dict, use_not_include, skip_render]

def load_scad_file_into_dict(resolved_scad, dest_namespace_dict, use_not_include, skip_render):
    #check the cache
    cached_dict = check_module_cache(resolved_scad, use_not_include)
    if cached_dict:
        dest_namespace_dict.update(cached_dict)
        return dest_namespace_dict

    #otherwise parse the file
    new_namespace_dict = get_scad_file_as_dict(resolved_scad)
    dest_namespace_dict.update(new_namespace_dict)

    #and update the cache
    update_module_cache(resolved_scad, new_namespace_dict, use_not_include, skip_render)

def load_scad_dir_into_dict(resolved_scad, dest_namespace_dict, use_not_include, skip_render):
    assert(resolved_scad.is_dir())

    #for each file in the dir
    for f in resolved_scad.iterdir():
        #skip non .scad files
        if f.suffix != ".scad":
            continue

        #load it
        subspace = ExpSolidNamespace(f)
        load_scad_file_or_dir_into_dict(f, subspace.__dict__, use_not_include, skip_render)

        #and add it to the dest_namespace_dict
        identifier = escape_openscad_identifier(f.stem)
        dest_namespace_dict[identifier] = subspace

def load_scad_file_or_dir_into_dict(filename, dest_namespace_dict, use_not_include, skip_render):
    assert(dest_namespace_dict != None)

    resolved_scad = resolve_scad_filename(filename)

    if not resolved_scad or not resolved_scad.exists():
        raise ValueError(f'Could not find .scad file {filename}.')

    if resolved_scad.is_file():
        load_scad_file_into_dict(resolved_scad,
                                 dest_namespace_dict,
                                 use_not_include,
                                 skip_render)
    elif resolved_scad.is_dir():
        load_scad_dir_into_dict(resolved_scad,
                                dest_namespace_dict,
                                use_not_include,
                                skip_render)

# use() & include() mimic OpenSCAD's use/include mechanics.
# -- use() makes methods in scad_file_path.scad available to be called.
# -- include() makes those methods available AND executes all code in
#    scad_file_path.scad, which may have side effects.
#    Unless you have a specific need, call use().
def get_callers_namespace_dict(depth=2):
    frame = inspect.currentframe()
    for i in range(depth):
        frame = frame.f_back

    if frame.f_code.co_name == "<module>":
        return frame.f_locals
    else:
        raise Exception("use & include can not be used inside a function!\n" +\
              "they would polute the modules namespace and only if executed. This\n" +\
              "has strange side effects! Use them on module level.\n")

        return frame.f_globals

def use(filename, skip_render=False):
    load_scad_file_or_dir_into_dict(filename, get_callers_namespace_dict(), True, skip_render)

def include(filename, skip_render=False):
    load_scad_file_or_dir_into_dict(filename, get_callers_namespace_dict(), False, skip_render)

def import_scad(filename, dest_namespace=None, use_not_include=True, skip_render=False):
    if dest_namespace == None:
        dest_namespace = ExpSolidNamespace(filename)

    load_scad_file_or_dir_into_dict(filename, dest_namespace.__dict__, use_not_include, skip_render)

    return dest_namespace

def extra_scad_include(filename, use_not_include=True):
    global extra_scad_includes
    extra_scad_includes.append((filename, use_not_include))

# ========================
# = our helper namespace =
# ========================
class ExpSolidNamespace(SimpleNamespace):
    """
    we create our own namespace class for imported openscad modules to be able
    to overwrite the __repr__ func. This enables in a python shell

        s = import_scad("...")
        print(s)

    to list the "contents" of this namespace
    """
    def __init__(self, filename):
        super().__init__()
        self.__filename__ = filename

    def __repr__(self):
        node_strings = []
        for k in sorted(self.__dict__):
            if not k.startswith("__"):
                i = self.__dict__[k]
                if isinstance(i, ExpSolidNamespace):
                    node_strings += [f'\t{k}']
                else:
                    node_strings += [f'\t{i.__doc__}']

        return '\n'.join([f'{self.__filename__}:'] + node_strings)




from solid2.core.object_base import OpenSCADObject, OpenSCADConstant
from solid2.core.scad_import import extra_scad_include
from pathlib import Path

extra_scad_include(f"{Path(__file__).parent.parent / '../libs/BOSL2/attachments.scad'}", use_not_include=False)

_tags = OpenSCADConstant('_tags')
_tag = OpenSCADConstant('_tag')
_tag_prefix = OpenSCADConstant('_tag_prefix')
_overlap = OpenSCADConstant('_overlap')
_color = OpenSCADConstant('_color')
_save_color = OpenSCADConstant('_save_color')
_attach_to = OpenSCADConstant('_attach_to')
_attach_anchor = OpenSCADConstant('_attach_anchor')
_attach_norot = OpenSCADConstant('_attach_norot')
_parent_anchor = OpenSCADConstant('_parent_anchor')
_parent_spin = OpenSCADConstant('_parent_spin')
_parent_orient = OpenSCADConstant('_parent_orient')
_parent_size = OpenSCADConstant('_parent_size')
_parent_geom = OpenSCADConstant('_parent_geom')
_tags_shown = OpenSCADConstant('_tags_shown')
_tags_hidden = OpenSCADConstant('_tags_hidden')
_ANCHOR_TYPES = OpenSCADConstant('_ANCHOR_TYPES')
EDGES_NONE = OpenSCADConstant('EDGES_NONE')
EDGES_ALL = OpenSCADConstant('EDGES_ALL')
EDGE_OFFSETS = OpenSCADConstant('EDGE_OFFSETS')
CORNERS_NONE = OpenSCADConstant('CORNERS_NONE')
CORNERS_ALL = OpenSCADConstant('CORNERS_ALL')
CORNER_OFFSETS = OpenSCADConstant('CORNER_OFFSETS')
class position(OpenSCADObject):
    def __init__(self, _from=None, **kwargs):
       super().__init__("position", {"_from" : _from, **kwargs})

class orient(OpenSCADObject):
    def __init__(self, dir=None, anchor=None, spin=None, **kwargs):
       super().__init__("orient", {"dir" : dir, "anchor" : anchor, "spin" : spin, **kwargs})

class attach(OpenSCADObject):
    def __init__(self, _from=None, to=None, overlap=None, norot=None, **kwargs):
       super().__init__("attach", {"_from" : _from, "to" : to, "overlap" : overlap, "norot" : norot, **kwargs})

class tag(OpenSCADObject):
    def __init__(self, tag=None, **kwargs):
       super().__init__("tag", {"tag" : tag, **kwargs})

class force_tag(OpenSCADObject):
    def __init__(self, tag=None, **kwargs):
       super().__init__("force_tag", {"tag" : tag, **kwargs})

class tag_scope(OpenSCADObject):
    def __init__(self, scope=None, **kwargs):
       super().__init__("tag_scope", {"scope" : scope, **kwargs})

class diff(OpenSCADObject):
    def __init__(self, remove=None, keep=None, **kwargs):
       super().__init__("diff", {"remove" : remove, "keep" : keep, **kwargs})

class tag_diff(OpenSCADObject):
    def __init__(self, tag=None, remove=None, keep=None, **kwargs):
       super().__init__("tag_diff", {"tag" : tag, "remove" : remove, "keep" : keep, **kwargs})

class intersect(OpenSCADObject):
    def __init__(self, intersect=None, keep=None, **kwargs):
       super().__init__("intersect", {"intersect" : intersect, "keep" : keep, **kwargs})

class tag_intersect(OpenSCADObject):
    def __init__(self, tag=None, intersect=None, keep=None, **kwargs):
       super().__init__("tag_intersect", {"tag" : tag, "intersect" : intersect, "keep" : keep, **kwargs})

class conv_hull(OpenSCADObject):
    def __init__(self, keep=None, **kwargs):
       super().__init__("conv_hull", {"keep" : keep, **kwargs})

class tag_conv_hull(OpenSCADObject):
    def __init__(self, tag=None, keep=None, **kwargs):
       super().__init__("tag_conv_hull", {"tag" : tag, "keep" : keep, **kwargs})

class hide(OpenSCADObject):
    def __init__(self, tags=None, **kwargs):
       super().__init__("hide", {"tags" : tags, **kwargs})

class show_only(OpenSCADObject):
    def __init__(self, tags=None, **kwargs):
       super().__init__("show_only", {"tags" : tags, **kwargs})

class show_all(OpenSCADObject):
    def __init__(self, **kwargs):
       super().__init__("show_all", {**kwargs})

class show_int(OpenSCADObject):
    def __init__(self, tags=None, **kwargs):
       super().__init__("show_int", {"tags" : tags, **kwargs})

class edge_mask(OpenSCADObject):
    def __init__(self, edges=None, _except=None, **kwargs):
       super().__init__("edge_mask", {"edges" : edges, "_except" : _except, **kwargs})

class corner_mask(OpenSCADObject):
    def __init__(self, corners=None, _except=None, **kwargs):
       super().__init__("corner_mask", {"corners" : corners, "_except" : _except, **kwargs})

class face_profile(OpenSCADObject):
    def __init__(self, faces=None, r=None, d=None, convexity=None, **kwargs):
       super().__init__("face_profile", {"faces" : faces, "r" : r, "d" : d, "convexity" : convexity, **kwargs})

class edge_profile(OpenSCADObject):
    def __init__(self, edges=None, _except=None, convexity=None, **kwargs):
       super().__init__("edge_profile", {"edges" : edges, "_except" : _except, "convexity" : convexity, **kwargs})

class corner_profile(OpenSCADObject):
    def __init__(self, corners=None, _except=None, r=None, d=None, convexity=None, **kwargs):
       super().__init__("corner_profile", {"corners" : corners, "_except" : _except, "r" : r, "d" : d, "convexity" : convexity, **kwargs})

class attachable(OpenSCADObject):
    def __init__(self, anchor=None, spin=None, orient=None, size=None, size2=None, shift=None, r=None, r1=None, r2=None, d=None, d1=None, d2=None, l=None, h=None, vnf=None, path=None, region=None, extent=None, cp=None, offset=None, anchors=None, two_d=None, axis=None, geom=None, **kwargs):
       super().__init__("attachable", {"anchor" : anchor, "spin" : spin, "orient" : orient, "size" : size, "size2" : size2, "shift" : shift, "r" : r, "r1" : r1, "r2" : r2, "d" : d, "d1" : d1, "d2" : d2, "l" : l, "h" : h, "vnf" : vnf, "path" : path, "region" : region, "extent" : extent, "cp" : cp, "offset" : offset, "anchors" : anchors, "two_d" : two_d, "axis" : axis, "geom" : geom, **kwargs})

class show_anchors(OpenSCADObject):
    def __init__(self, s=None, std=None, custom=None, **kwargs):
       super().__init__("show_anchors", {"s" : s, "std" : std, "custom" : custom, **kwargs})

class anchor_arrow(OpenSCADObject):
    def __init__(self, s=None, color=None, flag=None, _tag=None, **kwargs):
       super().__init__("anchor_arrow", {"s" : s, "color" : color, "flag" : flag, "_tag" : _tag, **kwargs})

class anchor_arrow2d(OpenSCADObject):
    def __init__(self, s=None, color=None, _tag=None, **kwargs):
       super().__init__("anchor_arrow2d", {"s" : s, "color" : color, "_tag" : _tag, **kwargs})

class expose_anchors(OpenSCADObject):
    def __init__(self, opacity=None, **kwargs):
       super().__init__("expose_anchors", {"opacity" : opacity, **kwargs})

class frame_ref(OpenSCADObject):
    def __init__(self, s=None, opacity=None, **kwargs):
       super().__init__("frame_ref", {"s" : s, "opacity" : opacity, **kwargs})

class _edges_text3d(OpenSCADObject):
    def __init__(self, txt=None, size=None, **kwargs):
       super().__init__("_edges_text3d", {"txt" : txt, "size" : size, **kwargs})

class _show_edges(OpenSCADObject):
    def __init__(self, edges=None, size=None, text=None, txtsize=None, toplabel=None, **kwargs):
       super().__init__("_show_edges", {"edges" : edges, "size" : size, "text" : text, "txtsize" : txtsize, "toplabel" : toplabel, **kwargs})

class _show_corners(OpenSCADObject):
    def __init__(self, corners=None, size=None, text=None, txtsize=None, toplabel=None, **kwargs):
       super().__init__("_show_corners", {"corners" : corners, "size" : size, "text" : text, "txtsize" : txtsize, "toplabel" : toplabel, **kwargs})

class _show_cube_faces(OpenSCADObject):
    def __init__(self, faces=None, size=None, toplabel=None, botlabel=None, **kwargs):
       super().__init__("_show_cube_faces", {"faces" : faces, "size" : size, "toplabel" : toplabel, "botlabel" : botlabel, **kwargs})

class reorient(OpenSCADObject):
    def __init__(self, anchor=None, spin=None, orient=None, size=None, size2=None, shift=None, r=None, r1=None, r2=None, d=None, d1=None, d2=None, l=None, h=None, vnf=None, path=None, region=None, extent=None, offset=None, cp=None, anchors=None, two_d=None, axis=None, geom=None, p=None, **kwargs):
       super().__init__("reorient", {"anchor" : anchor, "spin" : spin, "orient" : orient, "size" : size, "size2" : size2, "shift" : shift, "r" : r, "r1" : r1, "r2" : r2, "d" : d, "d1" : d1, "d2" : d2, "l" : l, "h" : h, "vnf" : vnf, "path" : path, "region" : region, "extent" : extent, "offset" : offset, "cp" : cp, "anchors" : anchors, "two_d" : two_d, "axis" : axis, "geom" : geom, "p" : p, **kwargs})

class named_anchor(OpenSCADObject):
    def __init__(self, name=None, pos=None, orient=None, spin=None, **kwargs):
       super().__init__("named_anchor", {"name" : name, "pos" : pos, "orient" : orient, "spin" : spin, **kwargs})

class attach_geom(OpenSCADObject):
    def __init__(self, size=None, size2=None, shift=None, scale=None, twist=None, r=None, r1=None, r2=None, d=None, d1=None, d2=None, l=None, h=None, vnf=None, region=None, extent=None, cp=None, offset=None, anchors=None, two_d=None, axis=None, **kwargs):
       super().__init__("attach_geom", {"size" : size, "size2" : size2, "shift" : shift, "scale" : scale, "twist" : twist, "r" : r, "r1" : r1, "r2" : r2, "d" : d, "d1" : d1, "d2" : d2, "l" : l, "h" : h, "vnf" : vnf, "region" : region, "extent" : extent, "cp" : cp, "offset" : offset, "anchors" : anchors, "two_d" : two_d, "axis" : axis, **kwargs})

class _attach_geom_2d(OpenSCADObject):
    def __init__(self, geom=None, **kwargs):
       super().__init__("_attach_geom_2d", {"geom" : geom, **kwargs})

class _attach_geom_size(OpenSCADObject):
    def __init__(self, geom=None, **kwargs):
       super().__init__("_attach_geom_size", {"geom" : geom, **kwargs})

class _attach_transform(OpenSCADObject):
    def __init__(self, anchor=None, spin=None, orient=None, geom=None, p=None, **kwargs):
       super().__init__("_attach_transform", {"anchor" : anchor, "spin" : spin, "orient" : orient, "geom" : geom, "p" : p, **kwargs})

class _get_cp(OpenSCADObject):
    def __init__(self, geom=None, **kwargs):
       super().__init__("_get_cp", {"geom" : geom, **kwargs})

class _force_anchor_2d(OpenSCADObject):
    def __init__(self, anchor=None, **kwargs):
       super().__init__("_force_anchor_2d", {"anchor" : anchor, **kwargs})

class _find_anchor(OpenSCADObject):
    def __init__(self, anchor=None, geom=None, **kwargs):
       super().__init__("_find_anchor", {"anchor" : anchor, "geom" : geom, **kwargs})

class _is_shown(OpenSCADObject):
    def __init__(self, **kwargs):
       super().__init__("_is_shown", {**kwargs})

class _standard_anchors(OpenSCADObject):
    def __init__(self, two_d=None, **kwargs):
       super().__init__("_standard_anchors", {"two_d" : two_d, **kwargs})

class _edges_vec_txt(OpenSCADObject):
    def __init__(self, x=None, **kwargs):
       super().__init__("_edges_vec_txt", {"x" : x, **kwargs})

class _edges_text(OpenSCADObject):
    def __init__(self, edges=None, **kwargs):
       super().__init__("_edges_text", {"edges" : edges, **kwargs})

class _is_edge_array(OpenSCADObject):
    def __init__(self, x=None, **kwargs):
       super().__init__("_is_edge_array", {"x" : x, **kwargs})

class _edge_set(OpenSCADObject):
    def __init__(self, v=None, **kwargs):
       super().__init__("_edge_set", {"v" : v, **kwargs})

class _normalize_edges(OpenSCADObject):
    def __init__(self, v=None, **kwargs):
       super().__init__("_normalize_edges", {"v" : v, **kwargs})

class _edges(OpenSCADObject):
    def __init__(self, v=None, _except=None, **kwargs):
       super().__init__("_edges", {"v" : v, "_except" : _except, **kwargs})

class _is_corner_array(OpenSCADObject):
    def __init__(self, x=None, **kwargs):
       super().__init__("_is_corner_array", {"x" : x, **kwargs})

class _normalize_corners(OpenSCADObject):
    def __init__(self, v=None, **kwargs):
       super().__init__("_normalize_corners", {"v" : v, **kwargs})

class _corner_set(OpenSCADObject):
    def __init__(self, v=None, **kwargs):
       super().__init__("_corner_set", {"v" : v, **kwargs})

class _corners(OpenSCADObject):
    def __init__(self, v=None, _except=None, **kwargs):
       super().__init__("_corners", {"v" : v, "_except" : _except, **kwargs})

class _corner_edges(OpenSCADObject):
    def __init__(self, edges=None, v=None, **kwargs):
       super().__init__("_corner_edges", {"edges" : edges, "v" : v, **kwargs})

class _corner_edge_count(OpenSCADObject):
    def __init__(self, edges=None, v=None, **kwargs):
       super().__init__("_corner_edge_count", {"edges" : edges, "v" : v, **kwargs})

class _corners_text(OpenSCADObject):
    def __init__(self, corners=None, **kwargs):
       super().__init__("_corners_text", {"corners" : corners, **kwargs})




from solid2.core.object_base import OpenSCADObject, OpenSCADConstant
from solid2.core.scad_import import extra_scad_include
from pathlib import Path

extra_scad_include(f"{Path(__file__).parent.parent / '../libs/BOSL2/constants.scad'}", use_not_include=False)

_UNDEF = OpenSCADConstant('_UNDEF')
INCH = OpenSCADConstant('INCH')
LEFT = OpenSCADConstant('LEFT')
RIGHT = OpenSCADConstant('RIGHT')
FRONT = OpenSCADConstant('FRONT')
FWD = OpenSCADConstant('FWD')
FORWARD = OpenSCADConstant('FORWARD')
BACK = OpenSCADConstant('BACK')
BOTTOM = OpenSCADConstant('BOTTOM')
BOT = OpenSCADConstant('BOT')
DOWN = OpenSCADConstant('DOWN')
TOP = OpenSCADConstant('TOP')
UP = OpenSCADConstant('UP')
CENTER = OpenSCADConstant('CENTER')
CTR = OpenSCADConstant('CTR')
CENTRE = OpenSCADConstant('CENTRE')
SEGMENT = OpenSCADConstant('SEGMENT')
RAY = OpenSCADConstant('RAY')
LINE = OpenSCADConstant('LINE')
IDENT = OpenSCADConstant('IDENT')
class get_slop(OpenSCADObject):
    def __init__(self, **kwargs):
       super().__init__("get_slop", {**kwargs})




from solid2.core.object_base import OpenSCADObject, OpenSCADConstant
from solid2.core.scad_import import extra_scad_include
from pathlib import Path

extra_scad_include(f"{Path(__file__).parent.parent / '../libs/BOSL2/trigonometry.scad'}", use_not_include=False)

class law_of_cosines(OpenSCADObject):
    def __init__(self, a=None, b=None, c=None, C=None, **kwargs):
       super().__init__("law_of_cosines", {"a" : a, "b" : b, "c" : c, "C" : C, **kwargs})

class law_of_sines(OpenSCADObject):
    def __init__(self, a=None, A=None, b=None, B=None, **kwargs):
       super().__init__("law_of_sines", {"a" : a, "A" : A, "b" : b, "B" : B, **kwargs})

class hyp_opp_to_adj(OpenSCADObject):
    def __init__(self, hyp=None, opp=None, **kwargs):
       super().__init__("hyp_opp_to_adj", {"hyp" : hyp, "opp" : opp, **kwargs})

class opp_hyp_to_adj(OpenSCADObject):
    def __init__(self, opp=None, hyp=None, **kwargs):
       super().__init__("opp_hyp_to_adj", {"opp" : opp, "hyp" : hyp, **kwargs})

class hyp_ang_to_adj(OpenSCADObject):
    def __init__(self, hyp=None, ang=None, **kwargs):
       super().__init__("hyp_ang_to_adj", {"hyp" : hyp, "ang" : ang, **kwargs})

class ang_hyp_to_adj(OpenSCADObject):
    def __init__(self, ang=None, hyp=None, **kwargs):
       super().__init__("ang_hyp_to_adj", {"ang" : ang, "hyp" : hyp, **kwargs})

class opp_ang_to_adj(OpenSCADObject):
    def __init__(self, opp=None, ang=None, **kwargs):
       super().__init__("opp_ang_to_adj", {"opp" : opp, "ang" : ang, **kwargs})

class ang_opp_to_adj(OpenSCADObject):
    def __init__(self, ang=None, opp=None, **kwargs):
       super().__init__("ang_opp_to_adj", {"ang" : ang, "opp" : opp, **kwargs})

class hyp_adj_to_opp(OpenSCADObject):
    def __init__(self, hyp=None, adj=None, **kwargs):
       super().__init__("hyp_adj_to_opp", {"hyp" : hyp, "adj" : adj, **kwargs})

class adj_hyp_to_opp(OpenSCADObject):
    def __init__(self, adj=None, hyp=None, **kwargs):
       super().__init__("adj_hyp_to_opp", {"adj" : adj, "hyp" : hyp, **kwargs})

class hyp_ang_to_opp(OpenSCADObject):
    def __init__(self, hyp=None, ang=None, **kwargs):
       super().__init__("hyp_ang_to_opp", {"hyp" : hyp, "ang" : ang, **kwargs})

class ang_hyp_to_opp(OpenSCADObject):
    def __init__(self, ang=None, hyp=None, **kwargs):
       super().__init__("ang_hyp_to_opp", {"ang" : ang, "hyp" : hyp, **kwargs})

class adj_ang_to_opp(OpenSCADObject):
    def __init__(self, adj=None, ang=None, **kwargs):
       super().__init__("adj_ang_to_opp", {"adj" : adj, "ang" : ang, **kwargs})

class ang_adj_to_opp(OpenSCADObject):
    def __init__(self, ang=None, adj=None, **kwargs):
       super().__init__("ang_adj_to_opp", {"ang" : ang, "adj" : adj, **kwargs})

class adj_opp_to_hyp(OpenSCADObject):
    def __init__(self, adj=None, opp=None, **kwargs):
       super().__init__("adj_opp_to_hyp", {"adj" : adj, "opp" : opp, **kwargs})

class opp_adj_to_hyp(OpenSCADObject):
    def __init__(self, opp=None, adj=None, **kwargs):
       super().__init__("opp_adj_to_hyp", {"opp" : opp, "adj" : adj, **kwargs})

class adj_ang_to_hyp(OpenSCADObject):
    def __init__(self, adj=None, ang=None, **kwargs):
       super().__init__("adj_ang_to_hyp", {"adj" : adj, "ang" : ang, **kwargs})

class ang_adj_to_hyp(OpenSCADObject):
    def __init__(self, ang=None, adj=None, **kwargs):
       super().__init__("ang_adj_to_hyp", {"ang" : ang, "adj" : adj, **kwargs})

class opp_ang_to_hyp(OpenSCADObject):
    def __init__(self, opp=None, ang=None, **kwargs):
       super().__init__("opp_ang_to_hyp", {"opp" : opp, "ang" : ang, **kwargs})

class ang_opp_to_hyp(OpenSCADObject):
    def __init__(self, ang=None, opp=None, **kwargs):
       super().__init__("ang_opp_to_hyp", {"ang" : ang, "opp" : opp, **kwargs})

class hyp_adj_to_ang(OpenSCADObject):
    def __init__(self, hyp=None, adj=None, **kwargs):
       super().__init__("hyp_adj_to_ang", {"hyp" : hyp, "adj" : adj, **kwargs})

class adj_hyp_to_ang(OpenSCADObject):
    def __init__(self, adj=None, hyp=None, **kwargs):
       super().__init__("adj_hyp_to_ang", {"adj" : adj, "hyp" : hyp, **kwargs})

class hyp_opp_to_ang(OpenSCADObject):
    def __init__(self, hyp=None, opp=None, **kwargs):
       super().__init__("hyp_opp_to_ang", {"hyp" : hyp, "opp" : opp, **kwargs})

class opp_hyp_to_ang(OpenSCADObject):
    def __init__(self, opp=None, hyp=None, **kwargs):
       super().__init__("opp_hyp_to_ang", {"opp" : opp, "hyp" : hyp, **kwargs})

class adj_opp_to_ang(OpenSCADObject):
    def __init__(self, adj=None, opp=None, **kwargs):
       super().__init__("adj_opp_to_ang", {"adj" : adj, "opp" : opp, **kwargs})

class opp_adj_to_ang(OpenSCADObject):
    def __init__(self, opp=None, adj=None, **kwargs):
       super().__init__("opp_adj_to_ang", {"opp" : opp, "adj" : adj, **kwargs})




from solid2.core.object_base import OpenSCADObject, OpenSCADConstant
from solid2.core.scad_import import extra_scad_include
from pathlib import Path

extra_scad_include(f"{Path(__file__).parent.parent / '../libs/BOSL2/skin.scad'}", use_not_include=False)

_MAP_DIAG = OpenSCADConstant('_MAP_DIAG')
_MAP_LEFT = OpenSCADConstant('_MAP_LEFT')
_MAP_UP = OpenSCADConstant('_MAP_UP')
class skin(OpenSCADObject):
    def __init__(self, profiles=None, slices=None, refine=None, method=None, sampling=None, caps=None, closed=None, z=None, style=None, convexity=None, anchor=None, cp=None, spin=None, orient=None, atype=None, **kwargs):
       super().__init__("skin", {"profiles" : profiles, "slices" : slices, "refine" : refine, "method" : method, "sampling" : sampling, "caps" : caps, "closed" : closed, "z" : z, "style" : style, "convexity" : convexity, "anchor" : anchor, "cp" : cp, "spin" : spin, "orient" : orient, "atype" : atype, **kwargs})

class linear_sweep(OpenSCADObject):
    def __init__(self, region=None, height=None, center=None, twist=None, scale=None, shift=None, slices=None, maxseg=None, style=None, convexity=None, cp=None, atype=None, h=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("linear_sweep", {"region" : region, "height" : height, "center" : center, "twist" : twist, "scale" : scale, "shift" : shift, "slices" : slices, "maxseg" : maxseg, "style" : style, "convexity" : convexity, "cp" : cp, "atype" : atype, "h" : h, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class spiral_sweep(OpenSCADObject):
    def __init__(self, poly=None, h=None, r=None, turns=None, higbee=None, center=None, r1=None, r2=None, d=None, d1=None, d2=None, higbee1=None, higbee2=None, internal=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("spiral_sweep", {"poly" : poly, "h" : h, "r" : r, "turns" : turns, "higbee" : higbee, "center" : center, "r1" : r1, "r2" : r2, "d" : d, "d1" : d1, "d2" : d2, "higbee1" : higbee1, "higbee2" : higbee2, "internal" : internal, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class path_sweep(OpenSCADObject):
    def __init__(self, shape=None, path=None, method=None, normal=None, closed=None, twist=None, twist_by_length=None, symmetry=None, last_normal=None, tangent=None, uniform=None, relaxed=None, caps=None, style=None, convexity=None, anchor=None, cp=None, spin=None, orient=None, atype=None, profiles=None, width=None, **kwargs):
       super().__init__("path_sweep", {"shape" : shape, "path" : path, "method" : method, "normal" : normal, "closed" : closed, "twist" : twist, "twist_by_length" : twist_by_length, "symmetry" : symmetry, "last_normal" : last_normal, "tangent" : tangent, "uniform" : uniform, "relaxed" : relaxed, "caps" : caps, "style" : style, "convexity" : convexity, "anchor" : anchor, "cp" : cp, "spin" : spin, "orient" : orient, "atype" : atype, "profiles" : profiles, "width" : width, **kwargs})

class path_sweep2d(OpenSCADObject):
    def __init__(self, profile=None, path=None, closed=None, caps=None, quality=None, style=None, convexity=None, anchor=None, cp=None, spin=None, orient=None, atype=None, **kwargs):
       super().__init__("path_sweep2d", {"profile" : profile, "path" : path, "closed" : closed, "caps" : caps, "quality" : quality, "style" : style, "convexity" : convexity, "anchor" : anchor, "cp" : cp, "spin" : spin, "orient" : orient, "atype" : atype, **kwargs})

class sweep(OpenSCADObject):
    def __init__(self, shape=None, transforms=None, closed=None, caps=None, style=None, convexity=None, anchor=None, cp=None, spin=None, orient=None, atype=None, **kwargs):
       super().__init__("sweep", {"shape" : shape, "transforms" : transforms, "closed" : closed, "caps" : caps, "style" : style, "convexity" : convexity, "anchor" : anchor, "cp" : cp, "spin" : spin, "orient" : orient, "atype" : atype, **kwargs})

class textured_linear_sweep(OpenSCADObject):
    def __init__(self, path=None, texture=None, tex_size=None, h=None, inset=None, rot=None, tscale=None, twist=None, scale=None, shift=None, style=None, reverse=None, l=None, counts=None, anchor=None, spin=None, orient=None, convexity=None, **kwargs):
       super().__init__("textured_linear_sweep", {"path" : path, "texture" : texture, "tex_size" : tex_size, "h" : h, "inset" : inset, "rot" : rot, "tscale" : tscale, "twist" : twist, "scale" : scale, "shift" : shift, "style" : style, "reverse" : reverse, "l" : l, "counts" : counts, "anchor" : anchor, "spin" : spin, "orient" : orient, "convexity" : convexity, **kwargs})

class textured_revolution(OpenSCADObject):
    def __init__(self, path=None, texture=None, tex_size=None, tscale=None, inset=None, rot=None, caps=None, wrap=None, shift=None, style=None, reverse=None, atype=None, convexity=None, counts=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("textured_revolution", {"path" : path, "texture" : texture, "tex_size" : tex_size, "tscale" : tscale, "inset" : inset, "rot" : rot, "caps" : caps, "wrap" : wrap, "shift" : shift, "style" : style, "reverse" : reverse, "atype" : atype, "convexity" : convexity, "counts" : counts, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class textured_cylinder(OpenSCADObject):
    def __init__(self, h=None, r=None, texture=None, tex_size=None, counts=None, tscale=None, inset=None, rot=None, style=None, reverse=None, shift=None, l=None, r1=None, r2=None, d=None, d1=None, d2=None, chamfer=None, chamfer1=None, chamfer2=None, rounding=None, rounding1=None, rounding2=None, convexity=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("textured_cylinder", {"h" : h, "r" : r, "texture" : texture, "tex_size" : tex_size, "counts" : counts, "tscale" : tscale, "inset" : inset, "rot" : rot, "style" : style, "reverse" : reverse, "shift" : shift, "l" : l, "r1" : r1, "r2" : r2, "d" : d, "d1" : d1, "d2" : d2, "chamfer" : chamfer, "chamfer1" : chamfer1, "chamfer2" : chamfer2, "rounding" : rounding, "rounding1" : rounding1, "rounding2" : rounding2, "convexity" : convexity, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class skin(OpenSCADObject):
    def __init__(self, profiles=None, slices=None, refine=None, method=None, sampling=None, caps=None, closed=None, z=None, style=None, anchor=None, cp=None, spin=None, orient=None, atype=None, **kwargs):
       super().__init__("skin", {"profiles" : profiles, "slices" : slices, "refine" : refine, "method" : method, "sampling" : sampling, "caps" : caps, "closed" : closed, "z" : z, "style" : style, "anchor" : anchor, "cp" : cp, "spin" : spin, "orient" : orient, "atype" : atype, **kwargs})

class linear_sweep(OpenSCADObject):
    def __init__(self, region=None, height=None, center=None, twist=None, scale=None, shift=None, slices=None, maxseg=None, style=None, cp=None, atype=None, h=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("linear_sweep", {"region" : region, "height" : height, "center" : center, "twist" : twist, "scale" : scale, "shift" : shift, "slices" : slices, "maxseg" : maxseg, "style" : style, "cp" : cp, "atype" : atype, "h" : h, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class _taperfunc(OpenSCADObject):
    def __init__(self, x=None, **kwargs):
       super().__init__("_taperfunc", {"x" : x, **kwargs})

class _ss_polygon_r(OpenSCADObject):
    def __init__(self, N=None, theta=None, **kwargs):
       super().__init__("_ss_polygon_r", {"N" : N, "theta" : theta, **kwargs})

class spiral_sweep(OpenSCADObject):
    def __init__(self, poly=None, h=None, r=None, turns=None, higbee=None, center=None, r1=None, r2=None, d=None, d1=None, d2=None, higbee1=None, higbee2=None, internal=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("spiral_sweep", {"poly" : poly, "h" : h, "r" : r, "turns" : turns, "higbee" : higbee, "center" : center, "r1" : r1, "r2" : r2, "d" : d, "d1" : d1, "d2" : d2, "higbee1" : higbee1, "higbee2" : higbee2, "internal" : internal, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class path_sweep(OpenSCADObject):
    def __init__(self, shape=None, path=None, method=None, normal=None, closed=None, twist=None, twist_by_length=None, symmetry=None, last_normal=None, tangent=None, uniform=None, relaxed=None, caps=None, style=None, transforms=None, anchor=None, cp=None, spin=None, orient=None, atype=None, **kwargs):
       super().__init__("path_sweep", {"shape" : shape, "path" : path, "method" : method, "normal" : normal, "closed" : closed, "twist" : twist, "twist_by_length" : twist_by_length, "symmetry" : symmetry, "last_normal" : last_normal, "tangent" : tangent, "uniform" : uniform, "relaxed" : relaxed, "caps" : caps, "style" : style, "transforms" : transforms, "anchor" : anchor, "cp" : cp, "spin" : spin, "orient" : orient, "atype" : atype, **kwargs})

class path_sweep2d(OpenSCADObject):
    def __init__(self, shape=None, path=None, closed=None, caps=None, quality=None, style=None, anchor=None, cp=None, spin=None, orient=None, atype=None, **kwargs):
       super().__init__("path_sweep2d", {"shape" : shape, "path" : path, "closed" : closed, "caps" : caps, "quality" : quality, "style" : style, "anchor" : anchor, "cp" : cp, "spin" : spin, "orient" : orient, "atype" : atype, **kwargs})

class _ofs_vmap(OpenSCADObject):
    def __init__(self, ofs=None, closed=None, **kwargs):
       super().__init__("_ofs_vmap", {"ofs" : ofs, "closed" : closed, **kwargs})

class _ofs_face_edge(OpenSCADObject):
    def __init__(self, face=None, firstlen=None, second=None, **kwargs):
       super().__init__("_ofs_face_edge", {"face" : face, "firstlen" : firstlen, "second" : second, **kwargs})

class sweep(OpenSCADObject):
    def __init__(self, shape=None, transforms=None, closed=None, caps=None, style=None, anchor=None, cp=None, spin=None, orient=None, atype=None, **kwargs):
       super().__init__("sweep", {"shape" : shape, "transforms" : transforms, "closed" : closed, "caps" : caps, "style" : style, "anchor" : anchor, "cp" : cp, "spin" : spin, "orient" : orient, "atype" : atype, **kwargs})

class subdivide_and_slice(OpenSCADObject):
    def __init__(self, profiles=None, slices=None, numpoints=None, method=None, closed=None, **kwargs):
       super().__init__("subdivide_and_slice", {"profiles" : profiles, "slices" : slices, "numpoints" : numpoints, "method" : method, "closed" : closed, **kwargs})

class slice_profiles(OpenSCADObject):
    def __init__(self, profiles=None, slices=None, closed=None, **kwargs):
       super().__init__("slice_profiles", {"profiles" : profiles, "slices" : slices, "closed" : closed, **kwargs})

class _closest_angle(OpenSCADObject):
    def __init__(self, alpha=None, beta=None, **kwargs):
       super().__init__("_closest_angle", {"alpha" : alpha, "beta" : beta, **kwargs})

class _smooth(OpenSCADObject):
    def __init__(self, data=None, len=None, closed=None, angle=None, **kwargs):
       super().__init__("_smooth", {"data" : data, "len" : len, "closed" : closed, "angle" : angle, **kwargs})

class rot_resample(OpenSCADObject):
    def __init__(self, rotlist=None, n=None, twist=None, scale=None, smoothlen=None, long=None, turns=None, closed=None, method=None, **kwargs):
       super().__init__("rot_resample", {"rotlist" : rotlist, "n" : n, "twist" : twist, "scale" : scale, "smoothlen" : smoothlen, "long" : long, "turns" : turns, "closed" : closed, "method" : method, **kwargs})

class _dp_distance_array(OpenSCADObject):
    def __init__(self, small=None, big=None, abort_thresh=None, **kwargs):
       super().__init__("_dp_distance_array", {"small" : small, "big" : big, "abort_thresh" : abort_thresh, **kwargs})

class _dp_distance_row(OpenSCADObject):
    def __init__(self, small=None, big=None, small_ind=None, tdist=None, **kwargs):
       super().__init__("_dp_distance_row", {"small" : small, "big" : big, "small_ind" : small_ind, "tdist" : tdist, **kwargs})

class _dp_extract_map(OpenSCADObject):
    def __init__(self, map=None, **kwargs):
       super().__init__("_dp_extract_map", {"map" : map, **kwargs})

class _skin_distance_match(OpenSCADObject):
    def __init__(self, poly1=None, poly2=None, **kwargs):
       super().__init__("_skin_distance_match", {"poly1" : poly1, "poly2" : poly2, **kwargs})

class _skin_aligned_distance_match(OpenSCADObject):
    def __init__(self, poly1=None, poly2=None, **kwargs):
       super().__init__("_skin_aligned_distance_match", {"poly1" : poly1, "poly2" : poly2, **kwargs})

class _skin_tangent_match(OpenSCADObject):
    def __init__(self, poly1=None, poly2=None, **kwargs):
       super().__init__("_skin_tangent_match", {"poly1" : poly1, "poly2" : poly2, **kwargs})

class _find_one_tangent(OpenSCADObject):
    def __init__(self, curve=None, edge=None, curve_offset=None, closed=None, **kwargs):
       super().__init__("_find_one_tangent", {"curve" : curve, "edge" : edge, "curve_offset" : curve_offset, "closed" : closed, **kwargs})

class associate_vertices(OpenSCADObject):
    def __init__(self, polygons=None, split=None, curpoly=None, **kwargs):
       super().__init__("associate_vertices", {"polygons" : polygons, "split" : split, "curpoly" : curpoly, **kwargs})

class _get_texture(OpenSCADObject):
    def __init__(self, tex=None, n=None, m=None, **kwargs):
       super().__init__("_get_texture", {"tex" : tex, "n" : n, "m" : m, **kwargs})

class textured_linear_sweep(OpenSCADObject):
    def __init__(self, path=None, texture=None, tex_size=None, h=None, counts=None, inset=None, rot=None, tscale=None, caps=None, col_wrap=None, twist=None, scale=None, shift=None, style=None, reverse=None, l=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("textured_linear_sweep", {"path" : path, "texture" : texture, "tex_size" : tex_size, "h" : h, "counts" : counts, "inset" : inset, "rot" : rot, "tscale" : tscale, "caps" : caps, "col_wrap" : col_wrap, "twist" : twist, "scale" : scale, "shift" : shift, "style" : style, "reverse" : reverse, "l" : l, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class textured_revolution(OpenSCADObject):
    def __init__(self, path=None, texture=None, tex_size=None, tscale=None, inset=None, rot=None, caps=None, wrap=None, shift=None, style=None, reverse=None, counts=None, **kwargs):
       super().__init__("textured_revolution", {"path" : path, "texture" : texture, "tex_size" : tex_size, "tscale" : tscale, "inset" : inset, "rot" : rot, "caps" : caps, "wrap" : wrap, "shift" : shift, "style" : style, "reverse" : reverse, "counts" : counts, **kwargs})

class textured_cylinder(OpenSCADObject):
    def __init__(self, h=None, r=None, texture=None, tex_size=None, counts=None, tscale=None, inset=None, rot=None, caps=None, style=None, reverse=None, shift=None, l=None, r1=None, r2=None, d=None, d1=None, d2=None, chamfer=None, chamfer1=None, chamfer2=None, rounding=None, rounding1=None, rounding2=None, **kwargs):
       super().__init__("textured_cylinder", {"h" : h, "r" : r, "texture" : texture, "tex_size" : tex_size, "counts" : counts, "tscale" : tscale, "inset" : inset, "rot" : rot, "caps" : caps, "style" : style, "reverse" : reverse, "shift" : shift, "l" : l, "r1" : r1, "r2" : r2, "d" : d, "d1" : d1, "d2" : d2, "chamfer" : chamfer, "chamfer1" : chamfer1, "chamfer2" : chamfer2, "rounding" : rounding, "rounding1" : rounding1, "rounding2" : rounding2, **kwargs})




#! /usr/bin/env python

# this is a basic "a mazing box" (mazebox), just tried to reimplement it with
# bosl2. It's based on https://www.thingiverse.com/thing:1481

from math import pi, cos
from solid2 import *
from solid2.extensions.bosl2 import cylindrical_extrude, zcyl, zrot, tube

def mazebox():
    #the maze is 64x90 pixel
    maze_surface = surface(file="maze7.png", center=True, invert=True)
    maze_projection = projection(cut=True) (maze_surface.up(10))

    #make the radius of the cylinder 3 times the maze width
    r = (64 * 3) / (2*pi)
    maze_mask = []
    segs = 10 #number of segments to get a "rounded" maze path

    for i in range(segs):
        maze_offset = maze_projection.offset(-1+cos(i/segs * pi /2))
        maze_mask += cylindrical_extrude(_or=r-(2*i/segs), ir=r-(2*(i+1)/segs)) (maze_offset)

    full_maze_mask = maze_mask + zrot(360/3)(maze_mask) + zrot(2*360/3)(maze_mask)

    maze_tube = tube(_or=r-0.2, ir=r-2.2, h=88, center=True)
    maze_tube_head = zcyl(r=r+5, h=10).down(40)

    return maze_tube + maze_tube_head - full_maze_mask

mazebox().save_as_scad()




#! /usr/bin/env python

from solid2 import *

# Since I don't like the OpenSCAD syntax I added the "access-style" syntax
# extension. This allows you to call all OpenSCAD builtin transformations, the
# convenience functions (up, down, left,...) and the modifiers (debug,
# background, ...) as if you would call a member function.
#
#       cube(10).up(5)
#
# is equivalent to
#
#       up(5)(cube(10))

c = cube(10, 20, 30).\
        down(5).\
            rotate(45, 45, 45)

s = ~sphere(10).forward(5)

(c-s).save_as_scad()

# generates the exactly the same code as 04-convenience.py:
#
#    // Generated by ExpSolidPython
#
#    difference() {
#            rotate(a = [45, 45, 45]) {
#                    translate(v = [0, 0, -5]) {
#                            cube(size = [10, 20, 30]);
#                    };
#            };
#            #translate(v = [0, 5, 0]) {
#                    sphere(r = 10);
#            };
#    };

